@@ -32,6 +32,7 @@ class MesosClient(object):
     UPDATE = 'UPDATE'
     FAILURE = 'FAILURE'
     RESCIND = 'RESCIND'
+    HEARTBEAT = 'HEARTBEAT'
 
     DISCONNECTED = 'DISCONNECTED'
     RECONNECTED = 'RECONNECTED'
@@ -51,6 +52,30 @@ class MesosClient(object):
             self.driver = None
             self.verify = False
 
+        def suppress(self):
+            '''
+            suppress framework
+            '''
+            headers = {
+                'Content-Type': 'application/json',
+                'Accept': 'application/json',
+                'Mesos-Stream-Id': self.streamId
+            }
+            suppress = {
+                "framework_id": {"value": self.frameworkId},
+                "type": "SUPPRESS"
+            }
+            try:
+                requests.post(
+                    self.mesos_url + '/api/v1/scheduler',
+                    json.dumps(suppress),
+                    headers=headers,
+                    auth=self.requests_auth,
+                    verify=self.verify
+                )
+            except Exception as e:
+                self.logger.error('Mesos:Suppress:Error:' + str(e))
+
         def tearDown(self):
             '''
             Undeclare framework
@@ -317,6 +342,7 @@ class MesosClient(object):
     def __init__(
             self,
             mesos_urls,
+            thread=True,
             frameworkId=None,
             frameworkName='Mesos HTTP framework',
             frameworkUser='root',
@@ -357,6 +383,7 @@ class MesosClient(object):
         self.streamId = None
         self.logger = logging.getLogger(__name__)
         self.stop = False
+        self.thread = thread
         self.disconnect = False
         self.callbacks = {
             MesosClient.SUBSCRIBED: [],
@@ -365,6 +392,7 @@ class MesosClient(object):
             MesosClient.ERROR: [],
             MesosClient.FAILURE: [],
             MesosClient.RESCIND: [],
+            MesosClient.HEARTBEAT: [],
             MesosClient.DISCONNECTED: [],
             MesosClient.RECONNECTED: []
         }
@@ -457,6 +485,8 @@ class MesosClient(object):
             self.callbacks[MesosClient.DISCONNECTED].append(callback)
         elif eventName == MesosClient.RECONNECTED:
             self.callbacks[MesosClient.RECONNECTED].append(callback)
+        elif eventName == MesosClient.HEARTBEAT:
+            self.callbacks[MesosClient.HEARTBEAT].append(callback)
         else:
             self.logger.error('No event %s' % (eventName))
             return False
@@ -474,6 +504,9 @@ class MesosClient(object):
     def __event_subscribed(self):
         return self.__event_callback(MesosClient.SUBSCRIBED, self.get_driver())
 
+    def __event_heartbeat(self, heartbeat):
+        return self.__event_callback(MesosClient.HEARTBEAT, heartbeat)
+
     def __event_disconnected(self):
         return self.__event_callback(MesosClient.DISCONNECTED, 'mesos master disconnected')
 
@@ -606,7 +639,6 @@ class MesosClient(object):
         return mesos_master
 
     def __register(self):
-        python_version = sys.version_info.major
         headers = {
             'Content-Type': 'application/json',
             'Accept': 'application/json'
@@ -657,7 +689,7 @@ class MesosClient(object):
                     if zk_url is None:
                         raise Exception('Could not detect master in zookeeper')
                     self.mesos_url = zk_url
-                self.logger.warn(
+                self.logger.warning(
                     'Try to connect to master: %s' % (self.mesos_url)
                 )
 
@@ -720,6 +752,14 @@ class MesosClient(object):
             self.__event_reconnected()
             self.disconnected = False
 
+        if self.thread == True:
+            self.__event_loop()
+        
+        if self.thread == False:
+            self.__event_single()
+
+    def __event_loop(self):
+        python_version = sys.version_info.major        
         first_line = True
         for line in self.long_pool.iter_lines():
             if self.stop or self.disconnect:
@@ -775,6 +815,9 @@ class MesosClient(object):
                     )
                     update_event.ack()
                     self.__event_update(mesos_update)
+                elif body['type'] == 'HEARTBEAT':
+                    self.logger.debug('Mesos:Heartbeat')
+                    self.__event_heartbeat(body['type'])
                 elif body['type'] == 'ERROR':
                     self.logger.error('Mesos:Error:' + body['error']['message'])
                     self.__event_error(body['error']['message'])
@@ -784,10 +827,8 @@ class MesosClient(object):
                     self.__event_callback(body['type'], body['message'])
                 elif body['type'] == 'FAILURE':
                     self.__event_callback(body['type'], body['failure'])
-                elif body['type'] == 'HEARTBEAT':
-                    self.logger.debug('Mesos:Heartbeat')
                 else:
-                    self.logger.warn(
+                    self.logger.warning(
                         '%s event no yet implemented' % (str(body['type']))
                     )
 
@@ -795,6 +836,69 @@ class MesosClient(object):
                     count_bytes = int(line[count_bytes:])
         return True
 
+    def __event_single(self):
+        python_version = sys.version_info.major        
+        body = self.long_pool.json()
+
+        self.logger.debug('Mesos:Event:%s' % (str(body['type'])))
+        self.logger.debug('Mesos:Message:' + str(body))
+        if body['type'] == 'SUBSCRIBED':
+            self.frameworkId = body['subscribed']['framework_id']['value']
+            self.logger.info(
+                'Mesos:Subscribe:Framework-Id:' + self.frameworkId
+            )
+            self.logger.info(
+                'Mesos:Subscribe:Stream-Id:' + self.streamId
+            )
+            if 'master_info' in body['subscribed']:
+                self.master_info = body['subscribed']['master_info']
+            self.__event_subscribed()
+        elif body['type'] == 'OFFERS':
+            mesos_offers = body['offers']['offers']
+            offers = []
+            for mesos_offer in mesos_offers:
+                offers.append(
+                    Offer(
+                        self.mesos_url,
+                        frameworkId=self.frameworkId,
+                        streamId=self.streamId,
+                        mesosOffer=mesos_offer,
+                        requests_auth=self.requests_auth,
+                        verify=self.verify
+                    )
+                )
+            self.__event_offers(offers)
+        elif body['type'] == 'UPDATE':
+            mesos_update = body['update']
+            update_event = Update(
+                self.mesos_url,
+                frameworkId=self.frameworkId,
+                streamId=self.streamId,
+                mesosUpdate=mesos_update,
+                requests_auth=self.requests_auth,
+                verify=self.verify 
+            )
+            update_event.ack()
+            self.__event_update(mesos_update)
+        elif body['type'] == 'ERROR':
+            self.logger.error('Mesos:Error:' + body['error']['message'])
+            self.__event_error(body['error']['message'])
+        elif body['type'] == 'RESCIND':
+            self.__event_callback(body['type'], body['rescind'])
+        elif body['type'] == 'MESSAGE':
+            self.__event_callback(body['type'], body['message'])
+        elif body['type'] == 'FAILURE':
+            self.__event_callback(body['type'], body['failure'])
+        elif body['type'] == 'HEARTBEAT':
+            self.logger.debug('Mesos:Heartbeat')
+            self.__event_heartbeat(body['type'])
+        else:
+            self.logger.warning(
+                '%s event no yet implemented' % (str(body['type']))
+            )    
+
+        return True
+
     def combine_offers(self, offers, operations, options=None):
         '''
         Accept offers with task operations
